#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
import ctypes
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')


class StorageControl():
    """
    <node>
        <interface name='com.kylin.kysdk.StorageControl'>

            <method name='SetStatus' >
                <arg type='i' direction='in' name='status'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

            <method name='GetStatus' >
                <arg type='i' direction='out' name='status'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        self.func_dict = {
            0: "DisableStorageDevices",
            1: "ReadOnly",
            2: "EnableStorageDevices"
        }

        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'storage-monitor','lv':'info','message':'{}'".format(
            time, name, ip, message)

        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")

    def SetStatus(self, status: int) -> int:
        """
        status
            0   禁用设备
            1   禁止写入
            2   不管控
        """
        if status in self.func_dict.keys():
            # 通过status调用对应函数
            ret = getattr(self, self.func_dict.get(status))()
            return ret
        else:
            return -1

    def GetStatus(self) -> int:
        try:
            file = open("/etc/mdm/storage", "r")
            value = file.read(1)
            file.close()
        except Exception:
            return 2
        return int(value)

    def DisableStorageDevices(self) -> int:
        try:
            self.log_transmit("开启存储设备禁用管控")
        except Exception:
            print("log message error")
        
        if self.GetStatus() == 0:
            return 0
        file = open("/etc/mdm/storage", "w")
        file.write("0")
        file.close()
        self.so.disabledevices(ctypes.c_char_p(b"08"))
        return 0

    def EnableStorageDevices(self) -> int:
        try:
            self.log_transmit("关闭存储设备管控")
        except Exception:
            print("log message error")

        if self.GetStatus() == 2:
            return 0

        file = open("/etc/mdm/storage", "w")
        file.write("2")
        file.close()

        self.so.set_up_usb()

        try:
            blacklist_f = open("/etc/mdm/black-list", "r")
            blacklist = blacklist_f.readlines()
            blacklist_f.close()
        except Exception:
            blacklist = []

        try:
            storagelist_f = open("/etc/mdm/disable-storage", "r")
            storagelist = storagelist_f.readlines()
            storagelist_f.close()
        except Exception:
            return 0

        for storage in storagelist:
            if storage not in blacklist:
                self.so.enabledevices(
                    ctypes.c_char_p(bytes(storage[0:4], encoding="utf8")),
                    ctypes.c_char_p(bytes(storage[4:8], encoding="utf8")))

        return 0

    def ReadOnly(self) -> int:
        try:
            self.log_transmit("开启存储设备只读管控")
        except Exception:
            print("log message error")
            
        if self.GetStatus() == 1:
            return 0

        file = open("/etc/mdm/storage", "w")
        file.write("1")
        file.close()

        self.so.set_up_usb()

        try:
            blacklist_f = open("/etc/mdm/black-list", "r")
            blacklist = blacklist_f.readlines()
            blacklist_f.close()
        except Exception:
            blacklist = []

        try:
            whitelist_f = open("/etc/mdm/white-list", "r")
            whitelist = whitelist_f.readlines()
            whitelist_f.close()
        except Exception:
            whitelist = []

        try:
            storagelist_f = open("/etc/mdm/disable-storage", "r")
            storagelist = set(storagelist_f.readlines())
            storagelist_f.close()
        except Exception:
            return 0

        for storage in storagelist:
            if storage not in blacklist:
                self.so.enabledevices(
                    ctypes.c_char_p(bytes(storage[0:4], encoding="utf8")),
                    ctypes.c_char_p(bytes(storage[4:8], encoding="utf8")))

        # time.sleep(2)
        # # 获取当前系统村存在的block设备
        # for device in self.context.list_devices(subsystem='block'):
        #     # 过滤调ata设备
        #     pid = subprocess.getoutput("udevadm info -a {} | grep idProduct | head -n 1 | tail -n 1".format(device.sys_path)).split("==")[-1].strip("\"")
        #     vid = subprocess.getoutput("udevadm info -a {} | grep idVendor | head -n 1 | tail -n 1".format(device.sys_path)).split("==")[-1].strip("\"")
        #     pidvid = pid + vid +"\n"
        #     if pidvid in whitelist:
        #         continue
        #     elif "/ata" not in device.sys_path and "/usb" in device.sys_path:
        #         dev_path = "/dev/" + device.sys_path.rsplit("/", 1)[-1]
        #         self.so.ReadOnly(ctypes.c_char_p(bytes(dev_path, encoding='utf8')))
        return 0


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.StorageControl", StorageControl())
    loop.run()
